<?php

namespace Drupal\blue_billywig\Plugin\Field\FieldWidget;

use Drupal\blue_billywig\BlueBillywigClient;
use Drupal\blue_billywig\Form\SettingsForm;
use Drupal\blue_billywig\Object\MediaClip;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\StringTextfieldWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'blue_billywig_id' widget.
 */
#[FieldWidget(
  id: 'blue_billywig_id',
  label: new TranslatableMarkup('Blue Billywig ID'),
  field_types: ['blue_billywig_id'],
)]
class BlueBillywigWidget extends StringTextfieldWidget {

  /**
   * The Blue Billywig client.
   *
   * @var \Drupal\blue_billywig\BlueBillywigClient
   */
  protected BlueBillywigClient $client;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->client = $container->get('blue_billywig.client');
    $instance->configFactory = $container->get('config.factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    // Only allow access to the element if the media is new. We do not allow
    // changing the ID of existing media items.
    if (!$items->getEntity()->isNew() && !empty($element['value']['#default_value'])) {
      $element['value']['#access'] = FALSE;
      // For existing media items, we show a preview of the media item.
      $playout_id = $this->configFactory->get(SettingsForm::CONFIG_NAME)->get('playout');
      $embed_type = $this->configFactory->get(SettingsForm::CONFIG_NAME)->get('embed_type');
      $element['preview'] = [
        '#markup' => Markup::create($this->client->embedCode($element['value']['#default_value'], $this->client->playouts()[$playout_id] ?? 'default', $embed_type)),
      ];
    }
    else {
      $element['value']['#access'] = FALSE;
      $element['file_upload'] = [
        '#type' => 'file',
        '#title' => $this->t('Upload a new video'),
        '#description' => $element['value']['#description'] ?? NULL,
        '#upload_validators' => [
          'FileExtension' => [
            'extensions' => 'mp4 wmv mov mkv avi',
          ],
        ],
      ];

      // Add a validator to verify if the file can be uploaded.
      $element['value']['#element_validate'][] = [static::class, 'uploadFile'];
    }

    return $element;
  }

  /**
   * Form validation handler for Blue Billywig ID elements.
   *
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function uploadFile(array $element, FormStateInterface $form_state): void {
    // Verify we have a valid field name.
    if (empty($element['#parents'][0])) {
      $form_state->setError($element, t('The file upload field is not properly configured.'));
      return;
    }

    // Fetch the path to the uploaded file.
    $uploaded_file = \Drupal::request()->files->get('files', [])[$element['#parents'][0]] ?? [];
    if (empty($uploaded_file)) {
      $form_state->setError($element, t('No file was uploaded.'));
      return;
    }

    // Temprarily move the uploaded file to a temp location using the filename.
    // This is necessary because the Blue Billywig SDK uses the filename from
    // the upload path and uses it to detect the file type.
    $tmp_path = 'temporary://' . $uploaded_file->getClientOriginalName();
    if (!\Drupal::service('file_system')->moveUploadedFile($uploaded_file->getRealPath(), $tmp_path)) {
      $form_state->setError($element, t('The file could not be uploaded. Please try again.'));
      return;
    }

    // Try to upload the file to the API and get the created Blue Billywig ID.
    $mediaclip = \Drupal::service('blue_billywig.client')->uploadFile($uploaded_file->getClientOriginalName(), \Drupal::service('file_system')->realpath($tmp_path));

    // Remove the temporary file after upload.
    \Drupal::service('file_system')->delete($tmp_path);

    if (!$mediaclip instanceof MediaClip) {
      $form_state->setError($element, t('The file could not be uploaded. Please try again.'));
      return;
    }

    // If the ID is valid, set it back to the element.
    $form_state->setValueForElement($element, $mediaclip->getId());
  }

}
